@@ -13,17 +13,19 @@ Versioning semantics:
 
 """
 
-__intname__ = 'ofunctions.file_utils'
-__author__ = 'Orsiris de Jong'
-__copyright__ = 'Copyright (C) 2017-2021 Orsiris de Jong'
-__description__ = 'Various file handling of which get_files_recursive is the most advanced'
-__licence__ = 'BSD 3 Clause'
-__version__ = '1.0.1'
-__build__ = '2021070101'
+__intname__ = "ofunctions.file_utils"
+__author__ = "Orsiris de Jong"
+__copyright__ = "Copyright (C) 2017-2022 Orsiris de Jong"
+__description__ = "File/dir/permissions/time handling"
+__licence__ = "BSD 3 Clause"
+__version__ = "1.2.1"
+__build__ = "2022041501"
+__compat__ = "python2.7+"
 
 import json
 import logging
 import os
+import sys
 import re
 import shutil
 from ofunctions import random
@@ -32,7 +34,31 @@ from datetime import datetime
 from fnmatch import fnmatch
 from itertools import chain
 from threading import Lock
-from typing import Callable, Iterable, Union, Optional
+
+# Python 2.7 compat fixes
+try:
+    from typing import Callable, Iterable, Union, Optional
+except ImportError:
+    pass
+if sys.version_info[0] < 3:
+
+    class FileNotFoundError(OSError):
+        pass
+
+
+# Python 2.7 compat where datetime.now() does not have .timestamp() method
+if sys.version_info[0] < 3 or sys.version_info[1] < 4:
+    # python version < 3.3
+    import time
+
+    def timestamp(date):
+        return time.mktime(date.timetuple())
+
+else:
+
+    def timestamp(date):
+        return date.timestamp()
+
 
 from command_runner import command_runner
 
@@ -59,7 +85,11 @@ def _file_lock():
         FILE_LOCK.release()
 
 
-def check_path_access(path: str, check: str = 'R') -> bool:
+def check_path_access(
+    path,  # type: str
+    check="R",  # type: str
+):
+    # type: (...) -> bool
     """
     Check if a path is accessible, if not, decompose path until we know which part isn't writable / readable
     when writable checks fail, we automatically fallback to readable tests
@@ -73,38 +103,45 @@ def check_path_access(path: str, check: str = 'R') -> bool:
     :param check: [R/W] check for readability / writability
     :return: bool: do we have desired access ?
     """
-    if check == 'W':
-        perm_type = 'writable'
+    if check == "W":
+        perm_type = "writable"
     else:
-        perm_type = 'readable'
+        perm_type = "readable"
 
     logger.debug('Checking access to path "{0}"'.format(path))
 
-    def _check_path_access(sub_path: str):
+    def _check_path_access(
+        sub_path,  # type: str
+    ):
+        # type: (...) -> bool
         if os.path.exists(sub_path):
-            obj = 'file' if os.path.isfile(sub_path) else 'directory'
-            if obj == 'file':
-                if check == 'W':
+            obj = "file" if os.path.isfile(sub_path) else "directory"
+            if obj == "file":
+                if check == "W":
                     try:
-                        fp = open(sub_path, 'a')
+                        fp = open(sub_path, "a")
                         fp.close()
                         res = True
                     except (PermissionError, OSError):
                         res = False
                 else:
                     try:
-                        fp = open(sub_path, 'r')
+                        fp = open(sub_path, "r")
                         fp.close()
                         res = True
                     except (PermissionError, OSError):
                         res = False
             # Handle directory tests
             else:
-                if check == 'W':
+                if check == "W":
                     try:
                         # Let's create a real file in path in order to check effective write permissions
-                        test_file = sub_path + '/.somehopefullyunexistenttestfile' + str(datetime.now().timestamp())
-                        open(test_file, 'w')
+                        test_file = (
+                            sub_path
+                            + "/.somehopefullyunexistenttestfile"
+                            + str(timestamp(datetime.now()))
+                        )
+                        open(test_file, "w")
                         remove_file(test_file)
                         res = True
                     except (IOError, OSError):
@@ -117,17 +154,28 @@ def check_path_access(path: str, check: str = 'R') -> bool:
                         res = False
 
             if res:
-                logger.debug('Path "{0}" is a {1} {2}.'.format(sub_path, perm_type, obj))
+                logger.debug(
+                    'Path "{0}" is a {1} {2}.'.format(sub_path, perm_type, obj)
+                )
             else:
-                logger.warning('Path "{0}" is a non {1} {2}.'.format(sub_path, perm_type, obj))
+                logger.warning(
+                    'Path "{0}" is a non {1} {2}.'.format(sub_path, perm_type, obj)
+                )
             return res
 
         else:
-            logger.warning('Path "{0}" does not exist or has ACLs that prevent access.'.format(sub_path))
+            logger.warning(
+                'Path "{0}" does not exist or has ACLs that prevent access.'.format(
+                    sub_path
+                )
+            )
         return False
 
-    def _split_path(path):
-        split_path = (path, '')
+    def _split_path(
+        path,  # type: str
+    ):
+        # type: (...) -> bool
+        split_path = (path, "")
         can_split = True
         failed_once = False
         while can_split is True:
@@ -136,15 +184,15 @@ def check_path_access(path: str, check: str = 'R') -> bool:
             else:
                 failed_once = True
             split_path = os.path.split(split_path[0])
-            if split_path[1] == '':
+            if split_path[1] == "":
                 can_split = False
         return failed_once
 
     result = _split_path(path)
-    if result and check == 'W':
+    if result and check == "W":
         # If not writable, fallback to a readable test
-        perm_type = 'readable'
-        check = 'R'
+        perm_type = "readable"
+        check = "R"
         _split_path(path)
 
     if result:
@@ -152,21 +200,30 @@ def check_path_access(path: str, check: str = 'R') -> bool:
     return not result
 
 
-def make_path(path: str):
+def make_path(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
         if not os.path.isdir(path):
             os.makedirs(path)
 
 
-def remove_file(path: str):
+def remove_file(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
         if os.path.isfile(path):
             os.remove(path)
 
 
-def remove_dir(path: str):
+def remove_dir(
+    path,  # type: str
+):
+    # type: (...) -> None
     with _file_lock():
         # May be false even if dir exists but ACLs deny
 
@@ -177,14 +234,22 @@ def remove_dir(path: str):
             shutil.rmtree(path)
 
 
-def move_file(source: str, dest: str):
+def move_file(
+    source,  # type: str
+    dest,  # type: str
+):
+    # type: (...) -> None
     make_path(os.path.dirname(dest))
     with _file_lock():
         # Using copy function because we don't want metadata, permissions, buffer nor anything else
         shutil.move(source, dest, copy_function=shutil.copy)
 
 
-def glob_path_match(path: str, pattern_list: list) -> bool:
+def glob_path_match(
+    path,  # type: str
+    pattern_list,  # type: list
+):
+    # type: (...) -> bool
     """
     Checks if path is in a list of glob style wildcard paths
     :param path: path of file / directory
@@ -194,21 +259,34 @@ def glob_path_match(path: str, pattern_list: list) -> bool:
     return any(fnmatch(path, pattern) for pattern in pattern_list)
 
 
-def log_perm_error(path: str) -> None:
+def log_perm_error(
+    path,  # type: str
+):
+    # type: (...) -> None
     """
     Default function that gets executed on get_paths_recursive permission error
     """
 
-    logger.warning('Non handled permission error on "{}"'.format(path))
+    logger.warning('Got permission error on "{}"'.format(path))
+    check_path_access(path, "W")
 
 
-def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list: list = None,
-                        d_include_list: list = None, f_include_list: list = None,
-                        exclude_dirs: bool = False, exclude_files: bool = False,
-                        ext_exclude_list: list = None, ext_include_list: list = None,
-                        min_depth: int = 1, max_depth: int = 0,
-                        primary_root: str = None, fn_on_perm_error: Callable = None,
-                        ) -> Union[Iterable, str]:
+def get_paths_recursive(
+    root,  # type: str
+    d_exclude_list=None,  # type: list
+    f_exclude_list=None,  # type: list
+    d_include_list=None,  # type: list
+    f_include_list=None,  # type: list
+    exclude_dirs=False,  # type: bool
+    exclude_files=False,  # type: bool
+    ext_exclude_list=None,  # type: list
+    ext_include_list=None,  # type: list
+    min_depth=1,  # type: int
+    max_depth=0,  # type: int
+    primary_root=None,  # type: str
+    fn_on_perm_error=None,  # type: Callable
+):
+    # type: (...) -> Union[Iterable, str]
     """
     Walk a path to recursively find files
     Accepts glob style windcards for every list parameter except file extension lists
@@ -249,7 +327,7 @@ def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list:
     if os.path.isdir(root):
         root = os.path.normpath(root)
     else:
-        raise FileNotFoundError('{} is not a directory.'.format(root))
+        raise FileNotFoundError("{} is not a directory.".format(root))
 
     # Check if we are allowed to read directory, if not, try to fix permissions if fn_on_perm_error is passed
     try:
@@ -271,7 +349,10 @@ def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list:
         # Let's also make sure that min_depth parameter is used as in gnu find
         min_depth = min_depth - 1
 
-    def _find_files(min_depth):
+    def _find_files(
+        min_depth,  # type: int
+    ):
+        # type: (...) -> Iterable
         if min_depth < 1:
             try:
                 if not exclude_dirs:
@@ -279,16 +360,30 @@ def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list:
                 if not exclude_files:
                     for file in os.listdir(root):
                         file_ext = os.path.splitext(file)[1]
-                        if os.path.isfile(os.path.join(root, file)) and \
-                                (not f_exclude_list or not glob_path_match(file, f_exclude_list)) and \
-                                (not ext_exclude_list or file_ext not in ext_exclude_list) and \
-                                (not f_include_list or glob_path_match(file, f_include_list)) and \
-                                (not ext_include_list or file_ext in ext_include_list):
+                        if (
+                            os.path.isfile(os.path.join(root, file))
+                            and (
+                                not f_exclude_list
+                                or not glob_path_match(file, f_exclude_list)
+                            )
+                            and (
+                                not ext_exclude_list or file_ext not in ext_exclude_list
+                            )
+                            and (
+                                not f_include_list
+                                or glob_path_match(file, f_include_list)
+                            )
+                            and (not ext_include_list or file_ext in ext_include_list)
+                        ):
                             yield os.path.join(root, file)
             except PermissionError:
                 pass
 
-    def _find_files_in_dirs(min_depth, max_depth):
+    def _find_files_in_dirs(
+        min_depth,  # type: int
+        max_depth,  # type: int
+    ):
+        # type: (...) -> Iterable
         min_depth = min_depth - 1
         if max_depth == 0 or max_depth > 1:
             max_depth = max_depth - 1 if max_depth > 1 else 0
@@ -298,23 +393,33 @@ def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list:
                     if os.path.isdir(d_full_path):
                         # p_root is the relative root the function has been called with recursively
                         # Let's check if p_root + d is in d_exclude_list
-                        p_root = os.path.join(primary_root, dir) if primary_root is not None else dir
-                        if (not d_exclude_list or not glob_path_match(p_root, d_exclude_list)) and \
-                           (not d_include_list or glob_path_match(p_root, d_include_list)):
-                            files_in_d = get_paths_recursive(d_full_path,
-                                                             d_exclude_list=d_exclude_list,
-                                                             f_exclude_list=f_exclude_list,
-                                                             ext_exclude_list=ext_exclude_list,
-                                                             ext_include_list=ext_include_list,
-                                                             d_include_list=d_include_list,
-                                                             f_include_list=f_include_list,
-                                                             exclude_dirs=exclude_dirs,
-                                                             exclude_files=exclude_files,
-                                                             min_depth=min_depth,
-                                                             max_depth=max_depth,
-                                                             primary_root=p_root,
-                                                             fn_on_perm_error=fn_on_perm_error,
-                                                             )
+                        p_root = (
+                            os.path.join(primary_root, dir)
+                            if primary_root is not None
+                            else dir
+                        )
+                        if (
+                            not d_exclude_list
+                            or not glob_path_match(p_root, d_exclude_list)
+                        ) and (
+                            not d_include_list
+                            or glob_path_match(p_root, d_include_list)
+                        ):
+                            files_in_d = get_paths_recursive(
+                                d_full_path,
+                                d_exclude_list=d_exclude_list,
+                                f_exclude_list=f_exclude_list,
+                                ext_exclude_list=ext_exclude_list,
+                                ext_include_list=ext_include_list,
+                                d_include_list=d_include_list,
+                                f_include_list=f_include_list,
+                                exclude_dirs=exclude_dirs,
+                                exclude_files=exclude_files,
+                                min_depth=min_depth,
+                                max_depth=max_depth,
+                                primary_root=p_root,
+                                fn_on_perm_error=fn_on_perm_error,
+                            )
                             for file in files_in_d:
                                 yield file
 
@@ -325,28 +430,44 @@ def get_paths_recursive(root: str, d_exclude_list: list = None, f_exclude_list:
     return chain(_find_files(min_depth), _find_files_in_dirs(min_depth, max_depth))
 
 
-def get_files_recursive(root: str, d_exclude_list: list = None, f_exclude_list: list = None,
-                        ext_exclude_list: list = None, ext_include_list: list = None,
-                        depth: int = 0, primary_root: str = None, fn_on_perm_error: Callable = None,
-                        include_dirs: bool = False) -> Union[Iterable, str]:
+def get_files_recursive(
+    root,  # type: str
+    d_exclude_list=None,  # type: list
+    f_exclude_list=None,  # type: list
+    ext_exclude_list=None,  # type: list
+    ext_include_list=None,  # type: list
+    depth=0,  # type: int
+    primary_root=None,  # type: str
+    fn_on_perm_error=None,  # type: Callable
+    include_dirs=False,  # type: bool
+):
+    # type: (...) -> Union[Iterable, str]
     """
     Wrapper for ofunctions.file_utils < 0.9.0 code
     """
 
-    return get_paths_recursive(root,
-                               d_exclude_list=d_exclude_list,
-                               f_exclude_list=f_exclude_list,
-                               exclude_dirs=not include_dirs,
-                               ext_exclude_list=ext_exclude_list,
-                               ext_include_list=ext_include_list,
-                               min_depth=1, max_depth=depth,
-                               primary_root=primary_root,
-                               fn_on_perm_error=fn_on_perm_error,
-                               )
+    return get_paths_recursive(
+        root,
+        d_exclude_list=d_exclude_list,
+        f_exclude_list=f_exclude_list,
+        exclude_dirs=not include_dirs,
+        ext_exclude_list=ext_exclude_list,
+        ext_include_list=ext_include_list,
+        min_depth=1,
+        max_depth=depth,
+        primary_root=primary_root,
+        fn_on_perm_error=fn_on_perm_error,
+    )
 
 
-def replace_in_file(source_file: str, text_to_search: str, replacement_text: str, dest_file: str = None,
-                    backup_ext: str = None) -> None:
+def replace_in_file(
+    source_file,  # type: str
+    text_to_search,  # type: str
+    replacement_text,  # type: str
+    dest_file=None,  # type: str
+    backup_ext=None,  # type: str
+):
+    # type: (...) -> None
     """
 
     :param source_file: source file to replace text in
@@ -356,7 +477,7 @@ def replace_in_file(source_file: str, text_to_search: str, replacement_text: str
     :param backup_ext: optional backup extension if no dest_file is given (inplace)
     :return:
     """
-    with open(source_file, 'r') as fp:
+    with open(source_file, "r") as fp:
         data = fp.read()
 
     if dest_file is not None:
@@ -364,18 +485,22 @@ def replace_in_file(source_file: str, text_to_search: str, replacement_text: str
     elif backup_ext is not None:
         file = source_file
         backup_file = source_file + backup_ext
-        with open(backup_file, 'w') as fp:
+        with open(backup_file, "w") as fp:
             fp.write(data)
     else:
         file = source_file
 
     data = data.replace(text_to_search, replacement_text)
 
-    with open(file, 'w') as fp:
+    with open(file, "w") as fp:
         fp.write(data)
 
 
-def get_file_time(path_to_file: str, mac_type: str = 'ctime') -> float:
+def get_file_time(
+    path_to_file,  # type: str
+    mac_type="ctime",  # type: str
+):
+    # type: (...) -> float
     """
     Returns file ctime/mtime/atime
 
@@ -388,37 +513,51 @@ def get_file_time(path_to_file: str, mac_type: str = 'ctime') -> float:
     Returned epochs are always UTC under Linux
     Returned epochs are TZ under Windows
     """
-    if mac_type not in ['ctime', 'mtime', 'atime']:
-        raise ValueError('Invalid file MAC time type request')
+    if mac_type not in ["ctime", "mtime", "atime"]:
+        raise ValueError("Invalid file MAC time type request")
     try:
-        return getattr(os.path, 'get' + mac_type)(path_to_file)
+        return getattr(os.path, "get" + mac_type)(path_to_file)
     # Some linuxes may not have os.path.getctime ?
     except AttributeError:
         stat = os.stat(path_to_file)
         try:
-            return getattr(stat, 'st_' + mac_type)
+            return getattr(stat, "st_" + mac_type)
         except AttributeError:
             # We're probably on Linux. No easy way to get creation dates here,
             # so we'll settle for when its content was last modified.
             return stat.st_mtime
 
 
-def file_creation_date(path_to_file: str) -> float:  # COMPAT <0.7.8
+def file_creation_date(
+    path_to_file,  # type: str
+):  # COMPAT <0.7.8
+    # type: (...) -> float
     """
     Wrapper for get_file_time (kept for compatibility reasons)
     """
-    return get_file_time(path_to_file, 'ctime')
+    return get_file_time(path_to_file, "ctime")
 
 
-def file_modification_date(path_to_file: str) -> float:  # COMPAT <0.7.8
+def file_modification_date(
+    path_to_file,  # type: str
+):  # COMPAT <0.7.8
+    # type: (...) -> float
     """
     Wrapper for get_file_time (kept for compatibility reasons)
     """
-    return get_file_time(path_to_file, 'mtime')
+    return get_file_time(path_to_file, "mtime")
 
 
-def check_file_timestamp_delta(file: str, mac_type: str = 'ctime', years: int = 0, days: int = 0, hours: int = 0,
-                       minutes: int = 0, seconds: int = 0) -> bool:
+def check_file_timestamp_delta(
+    file,  # type: str,
+    mac_type="ctime",  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):
+    # type: (...) -> bool
     """
     mac_type can be ctime, mtime, or atime for comparison purposes
 
@@ -430,84 +569,146 @@ def check_file_timestamp_delta(file: str, mac_type: str = 'ctime', years: int =
 
     """
     if not os.path.isfile(file):
-        raise FileNotFoundError('[%s] not found.' % file)
-    delta = seconds + (minutes * 60) + (hours * 3600) + (days * 86400) + (years * 31536000)
+        raise FileNotFoundError("[%s] not found." % file)
+    delta = (
+        seconds + (minutes * 60) + (hours * 3600) + (days * 86400) + (years * 31536000)
+    )
     # file creation date is UTC for Linux, TZ for Windows
-    if os.name == 'nt':
-        now = datetime.now().timestamp()
+    if os.name == "nt":
+        now = timestamp(datetime.now())
     else:
-        now = datetime.utcnow().timestamp()
+        now = timestamp(datetime.utcnow())
     return bool((now + delta - get_file_time(file, mac_type)) > 0)
 
 
-def is_file_older_than(file: str, years: int = 0, days: int = 0, hours: int = 0,
-                       minutes: int = 0, seconds: int = 0) -> bool:  # COMPAT <0.7.8
+def is_file_older_than(
+    file,  # type: str,
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):  # COMPAT <0.7.8
+    # type: (...) -> bool
     """
     Wrapper for check_file_times kept for compatibility
     """
-    mac_type = 'ctime'
-    return check_file_timestamp_delta(file, mac_type='ctime', years=-years, days=-days,
-                                      hours=-hours, minutes=-minutes, seconds=-seconds)
+    mac_type = "ctime"
+    return check_file_timestamp_delta(
+        file,
+        mac_type="ctime",
+        years=-years,
+        days=-days,
+        hours=-hours,
+        minutes=-minutes,
+        seconds=-seconds,
+    )
 
 
-def remove_files_on_timestamp_delta(directory: str, mac_type: str = 'ctime', years: int = 0, days: int = 0,
-                                    hours: int = 0, minutes: int = 0, seconds: int = 0) -> None:
+def remove_files_on_timestamp_delta(
+    directory,  # type: str,
+    mac_type="ctime",  # type: str
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):
+    # type: (...) -> None
     """
     Remove files older than given delta from now
     """
 
     if not os.path.isdir(directory):
-        raise FileNotFoundError('[%s] not found.' % directory)
+        raise FileNotFoundError("[%s] not found." % directory)
 
     for _, _, filenames in os.walk(directory):
         for filename in filenames:
             filename = os.path.join(directory, filename)
             try:
-                if check_file_timestamp_delta(filename, mac_type=mac_type, years=years, days=days,
-                                              hours=hours, minutes=minutes, seconds=seconds):
+                if check_file_timestamp_delta(
+                    filename,
+                    mac_type=mac_type,
+                    years=years,
+                    days=days,
+                    hours=hours,
+                    minutes=minutes,
+                    seconds=seconds,
+                ):
                     os.remove(filename)
             except FileNotFoundError:
                 pass
             except (IOError, OSError):
-                raise OSError('Cannot remove file [%s].' % filename)
+                raise OSError("Cannot remove file [%s]." % filename)
 
 
-def remove_files_older_than(directory: str, years: int = 0, days: int = 0, hours: int = 0,
-                            minutes: int = 0, seconds: int = 0) -> None:  # COMPAT <0.7.8
+def remove_files_older_than(
+    directory,  # type: str,
+    years=0,  # type: int
+    days=0,  # type: int
+    hours=0,  # type: int
+    minutes=0,  # type: int
+    seconds=0,  # type: int
+):  # COMPAT <0.7.8
+    # type: (...) -> None
     """
     Wrapper for remove_files_on_timestamp_delta for compatibility
     """
-    mac_type = 'ctime'
-    remove_files_on_timestamp_delta(directory, mac_type='ctime',
-                                    years=-years, days=-days,
-                                    hours=-hours, minutes=-minutes, seconds=-seconds)
+    mac_type = "ctime"
+    remove_files_on_timestamp_delta(
+        directory,
+        mac_type="ctime",
+        years=-years,
+        days=-days,
+        hours=-hours,
+        minutes=-minutes,
+        seconds=-seconds,
+    )
 
 
-def remove_bom(file: str) -> None:
+def remove_bom(
+    file,  # type: str
+):
+    # type: (...) -> None
     """
     Remove BOM from existing UTF-8 file
     We don't use any utf-8-sig codec magic here to avoid any UnicodeDecodeErrors
     hence the function is uglier than it should, but less error prone
     """
+
+    buffer = 32768
+
     try:
-        with open(file, 'rb') as file_handle_in:
+        with open(file, "rb") as file_handle_in:
             data = file_handle_in.read(3)
             # Throw away the data if it's a BOM
-            if data == b'\xef\xbb\xbf':
-                data = file_handle_in.read(32768)
+            if data == b"\xef\xbb\xbf":
+                data = file_handle_in.read(buffer)
             else:
                 return
-            with open(file + '.tmp', 'wb') as file_handle_out:
+            with open(file + ".tmp", "wb") as file_handle_out:
                 while len(data) > 0:
                     file_handle_out.write(data)
-                    data = file_handle_in.read(32768)
-        if os.path.isfile(file + '.tmp'):
-            os.replace(file + '.tmp', file)
-    except Exception:
-        raise OSError
+                    data = file_handle_in.read(buffer)
+        if os.path.isfile(file + ".tmp"):
+            # Python 2.7 does not have os.replace()
+            try:
+                os.replace(file + ".tmp", file)
+            except AttributeError:  # Python 2 workaround
+                try:
+                    os.remove(file)
+                except:
+                    pass
+                os.rename(file + ".tmp", file)
+    except Exception as exc:
+        raise OSError(exc)
 
 
-def write_json_to_file(file: str, data: Union[dict, list]):
+def write_json_to_file(
+    file,  # type: str
+    data,  # type: Union[dict, list]
+):
+    # type: (...) -> None
     """
     Creates a manifest to the file containing it's sha256sum and the installation result
 
@@ -516,11 +717,14 @@ def write_json_to_file(file: str, data: Union[dict, list]):
     :return:
     """
 
-    with open(file, 'w', encoding='utf-8') as file_handle:
+    with open(file, "w", encoding="utf-8") as file_handle:
         json.dump(data, file_handle, ensure_ascii=False)
 
 
-def read_json_from_file(file: str) -> dict:
+def read_json_from_file(
+    file,  # type: str
+):
+    # type: (...) -> dict
     """
     Verifies exit code of the file manifest and returns True if installed successfully
     :param file: (str) path to file
@@ -528,56 +732,91 @@ def read_json_from_file(file: str) -> dict:
     """
 
     if os.path.isfile(file):
-        with open(file, 'r', encoding='utf-8') as file_handle:
+        with open(file, "r", encoding="utf-8") as file_handle:
             file_content = json.load(file_handle)
             return file_content
     else:
         return {}
 
 
-def grep(file: str, pattern: str) -> list:
+def grep(
+    file,  # type: str
+    pattern,  # type: str
+    ignorecase=False,  # type bool
+):
+    # type: (...) -> list
+    """
+    Grep emulation
+    """
     if not os.path.isfile(file):
         raise FileNotFoundError(file)
     result = []
-    with open(file, 'r') as file_handle:
+
+    with open(file, "r") as file_handle:
         for line in file_handle:
-            if re.search(pattern, line):
-                result.append(line)
+            if ignorecase:
+                if re.search(pattern, line, re.IGNORECASE):
+                    result.append(line)
+            else:
+                if re.search(pattern, line):
+                    result.append(line)
     return result
 
 
-def hide_windows_file(file: str, hidden: bool = True) -> bool:
-    result, _ = command_runner('attrib %sh "%s"' % ('+' if hidden else '-', file))
+def hide_windows_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides / unhides a file under windows by using attrib command
+    """
+    result, _ = command_runner('attrib %sh "%s"' % ("+" if hidden else "-", file))
     if result == 0:
         return True
     return False
 
 
-def hide_unix_file(file: str, hidden: bool = True) -> bool:
-    if (file.startswith('.') and hidden) or (not file.startswith('.') and not hidden):
+def hide_unix_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides / unhides a file under unix by prepending a dot
+    """
+    if (file.startswith(".") and hidden) or (not file.startswith(".") and not hidden):
         return True
 
     file_directory = os.path.dirname(file)
     filename = os.path.basename(file)
 
     try:
-        if file.startswith('.') and not hidden:
-            move_file(file, os.path.join(file_directory, '.{}'.format(filename)))
-        if not file.startswith('.') and hidden:
-            move_file(file, os.path.join(file_directory, filename.lstrip('.')))
+        if file.startswith(".") and not hidden:
+            move_file(file, os.path.join(file_directory, ".{}".format(filename)))
+        if not file.startswith(".") and hidden:
+            move_file(file, os.path.join(file_directory, filename.lstrip(".")))
         return True
     except OSError:
         return False
 
 
-def hide_file(file: str, hidden: bool = True) -> bool:
-    if os.name == 'nt':
+def hide_file(
+    file,  # type: str
+    hidden=True,  # type: bool
+):
+    # type: (...) -> bool
+    """
+    Hides/unhindes a file under Windows / Unix platforms
+    """
+    if os.name == "nt":
         return hide_windows_file(file, hidden)
     else:
         return hide_unix_file(file, hidden)
 
 
-def get_writable_temp_dir() -> Optional[str]:
+def get_writable_temp_dir():
+    # type: (...) -> Optional[str]
     """
     Try to find a writable temporary directory
     """
@@ -585,31 +824,38 @@ def get_writable_temp_dir() -> Optional[str]:
 
     candidate_list = [
         # POSIX correct variable
-        os.environ.get('TMPDIR', False) if os_name != 'nt' else None,
-        os.environ.get('TEMP', False),
-        os.environ.get('TMP', False),
-        os.path.join(os.environ.get('SYSTEMROOT'), 'Temp') if os_name == 'nt' else None,
-        '/tmp' if os_name != 'nt' else None,
-        '/var/tmp' if os_name != 'nt' else None,
+        os.environ.get("TMPDIR", False) if os_name != "nt" else None,
+        os.environ.get("TEMP", False),
+        os.environ.get("TMP", False),
+        os.path.join(os.environ.get("SYSTEMROOT"), "Temp") if os_name == "nt" else None,
+        "/tmp" if os_name != "nt" else None,
+        "/var/tmp" if os_name != "nt" else None,
     ]
 
     for candidate in candidate_list:
-        if candidate and check_path_access(candidate, 'W'):
+        if candidate and check_path_access(candidate, "W"):
             return candidate
     return None
 
 
-def get_writable_random_file(ident_str: str = 'tmp_file_utils') -> Optional[str]:
+def get_writable_random_file(
+    ident_str="tmp_file_utils",  # type: str
+):
+    # type: (...) -> Optional[str]
     """
     Try to return a path to a not yet existing random file
     """
 
-    timestamp_format = '%Y-%m-%d.%H-%M-%S.%f'
+    timestamp_format = "%Y-%m-%d.%H-%M-%S.%f"
 
     tmp_dir = get_writable_temp_dir()
     if tmp_dir:
-        return os.path.join(tmp_dir, '{}.{}.{}.tmp'.format(ident_str,
-                                                          datetime.utcnow().strftime(timestamp_format),
-                                                          random.random_string(16),
-                                                          ))
+        return os.path.join(
+            tmp_dir,
+            "{}.{}.{}.tmp".format(
+                ident_str,
+                datetime.utcnow().strftime(timestamp_format),
+                random.random_string(16),
+            ),
+        )
     return None
