@@ -15,36 +15,63 @@ Versioning semantics:
 
 __intname__ = "ofunctions.logger_utils"
 __author__ = "Orsiris de Jong"
-__copyright__ = "Copyright (C) 2014-2021 Orsiris de Jong"
+__copyright__ = "Copyright (C) 2014-2022 Orsiris de Jong"
 __description__ = "Shorthand for logger initialization, recording worst called loglevel and handling nice console output"
 __licence__ = "BSD 3 Clause"
-__version__ = "2.0.2"
-__build__ = "2021080201"
+__version__ = "2.2.0"
+__build__ = "2022041602"
+__compat__ = "python2.7+"
 
 import logging
 import os
 import sys
 import tempfile
 from logging.handlers import RotatingFileHandler
-from typing import Union, Tuple
+
+# python 2.7 compat fixes
+try:
+    from typing import Union, Tuple
+except ImportError:
+    pass
 
 # Logging functions ########################################################
 
-FORMATTER = logging.Formatter(u"%(asctime)s :: %(levelname)s :: %(message)s")
+FORMATTER = logging.Formatter("%(asctime)s :: %(levelname)s :: %(message)s")
 MP_FORMATTER = logging.Formatter(
-    u"%(asctime)s :: %(levelname)s :: %(processName)s :: %(message)s"
+    "%(asctime)s :: %(levelname)s :: %(processName)s :: %(message)s"
 )
 
 
-# Allows to change default logging output or record events
+class FixPython2Logging(logging.Filter):
+    def __init__(self):
+        self._worst_level = logging.INFO
+        if sys.version_info[0] < 3:
+            # pylint: disable=E1003 (bad-super-call)
+            super(logging.Filter, self).__init__()
+        else:
+            super().__init__()
+
+    def filter(self, record):
+        # type: (str) -> bool
+        # Fix python2 unicodedecodeerrors when non unicode strings are sent to logger
+        if sys.version_info[0] < 3:
+            record.msg = safe_string_convert(record.msg)
+        return True
+
+
 class ContextFilterWorstLevel(logging.Filter):
     """
     This class records the worst loglevel that was called by logger
+    Allows to change default logging output or record events
     """
 
     def __init__(self):
         self._worst_level = logging.INFO
-        super().__init__()
+        if sys.version_info[0] < 3:
+            # pylint: disable=E1003 (bad-super-call)
+            super(logging.Filter, self).__init__()
+        else:
+            super().__init__()
 
     @property
     def worst_level(self):
@@ -54,11 +81,13 @@ class ContextFilterWorstLevel(logging.Filter):
         return self._worst_level
 
     @worst_level.setter
-    def worst_level(self, value: int):
+    def worst_level(self, value):
+        # type: (int) -> None
         if isinstance(value, int):
             self._worst_level = value
 
-    def filter(self, record) -> bool:
+    def filter(self, record):
+        # type: (str) -> bool
         """
         A filter can change the default log output
         This one simply records the worst log level called
@@ -73,8 +102,9 @@ class ContextFilterWorstLevel(logging.Filter):
 
 
 def logger_get_console_handler(
-    multiprocessing_formatter: bool = False,
-) -> Union[logging.StreamHandler, None]:
+    multiprocessing_formatter=False,
+):
+    # type: (bool) -> Union[logging.StreamHandler, None]
     """
     Returns a console handler that outputs as UTF-8 regardless of the platform
     """
@@ -95,7 +125,9 @@ def logger_get_console_handler(
             # import codecs
             # sys.stdout = codecs.getwriter('utf-8')(sys.stdout)
         except AttributeError:
-            print("Cannot force console encoding.")
+            pass
+            # print("Cannot force console encoding.")
+            # Python2 does not have ssys.stdout.reconfigure
             # IPython interpreter does not know about sys.stdout.reconfigure function
             # Neither does it now detach or fileno()
             # sys.stdout = open(sys.stdout.fileno(), mode='w', encoding='utf-8', errors='backslashreplace', buffering=1)
@@ -117,9 +149,8 @@ def logger_get_console_handler(
         return console_handler
 
 
-def logger_get_file_handler(
-    log_file: str, multiprocessing_formatter: bool = False
-) -> Tuple[Union[RotatingFileHandler, None], Union[str, None]]:
+def logger_get_file_handler(log_file, multiprocessing_formatter=False):
+    # type: (str, bool) -> Tuple[Union[RotatingFileHandler, None], Union[str, None]]
     """
     Returns a log file handler
     On failire, will return a temporary file log handler
@@ -133,7 +164,7 @@ def logger_get_file_handler(
         file_handler = RotatingFileHandler(
             log_file, mode="a", encoding="utf-8", maxBytes=1048576, backupCount=3
         )
-    except OSError as exc:
+    except (OSError, IOError) as exc:
         try:
             print(
                 "Cannot create logfile. Trying to obtain temporary log file.\nMessage: %s"
@@ -165,17 +196,17 @@ def logger_get_file_handler(
 
 
 def logger_get_logger(
-    log_file: str = None,
-    temp_log_file: str = None,
-    console: bool = True,
-    debug: bool = False,
-    multiprocessing_formatter: bool = False,
-) -> logging.Logger:
+    log_file=None,  # type: str
+    temp_log_file=None,  # type: str
+    console=True,  # type: bool
+    debug=False,  # type: bool
+    multiprocessing_formatter=False,  # type: bool
+):
+    # type: (...) -> logging.Logger
     """
     Returns a logger instance, just as logger.getLogger(), configured for console and/or file
     """
     # If a name is given to getLogger, than modules can't log to the root logger
-    log_filter = ContextFilterWorstLevel()
     _logger = logging.getLogger()
 
     # Remove earlier handlers if exist
@@ -183,7 +214,8 @@ def logger_get_logger(
         _logger.handlers.pop()
 
     # Add context filter
-    _logger.addFilter(log_filter)
+    _logger.addFilter(FixPython2Logging())
+    _logger.addFilter(ContextFilterWorstLevel())
 
     if debug:
         _logger.setLevel(logging.DEBUG)
@@ -230,3 +262,40 @@ def logger_get_logger(
                 )
     _logger.propagate = True
     return _logger
+
+
+def safe_string_convert(string):
+    """
+    Allows to encode strings for hacky UTF-8 logging in python 2.7
+    """
+    try:
+        return string.decode("utf8")
+    except Exception:  # noqa
+        try:
+            return string.decode("unicode-escape")
+        except Exception:  # noqa
+            try:
+                return string.decode("latin1")
+            except Exception:  # noqa
+                if sys.version_info[0] < 3:
+                    # pylint: disable=E0602 (undefined-variable)
+                    if isinstance(string, unicode):  # noqa
+                        return string
+                try:
+                    return (
+                        b"logger_utils: Cannot convert logged string. Passing it as binary blob: "
+                        + bytes(string)
+                    )
+                except Exception:  # noqa
+                    return string
+
+
+def get_worst_logger_level(_logger):
+    # type (logging.Logger) -> int
+    """
+    Return the worst log level called
+    """
+    for flt in _logger.filters:
+        if isinstance(flt, ContextFilterWorstLevel):
+            return flt.worst_level
+    return 0
