@@ -6,7 +6,7 @@ from multiprocessing import current_process
 import multiprocessing_logging
 from functools import wraps
 
-__version__ = '0.1.0'
+__version__ = "0.1.0"
 
 logger = logging.getLogger()
 
@@ -20,7 +20,7 @@ def logs_mp_process_names(fn):
                 process_name = current_process().name
             except BaseException:
                 process_name = __name__
-            record.msg = f'{process_name} :: {record.msg}'
+            record.msg = f"{process_name} :: {record.msg}"
             return True
 
     multiprocessing_logging.install_mp_handler()
@@ -33,4 +33,4 @@ def logs_mp_process_names(fn):
         logger.addFilter(f)
         return fn(*args, **kwargs)
 
-    return wrapper
\ No newline at end of file
+    return wrapper
