@@ -13,20 +13,26 @@ Versioning semantics:
 
 """
 
+# python 2.7 compat fixes
+from __future__ import unicode_literals
+
 __intname__ = "ofunctions.string_handling"
 __author__ = "Orsiris de Jong"
-__copyright__ = "Copyright (C) 2014-2021 Orsiris de Jong"
+__copyright__ = "Copyright (C) 2014-2022 Orsiris de Jong"
 __description__ = "Simple string sanitization functions"
 __licence__ = "BSD 3 Clause"
-__version__ = "0.1.2"
-__build__ = "2021032901"
+__version__ = "1.1.1"
+__build__ = "2022041601"
+__compat__ = "python2.7+"
 
 
+import sys
 import unicodedata
 import re
 
 
-def convert_accents(string: str) -> str:
+def convert_accents(string):
+    # type: (str) -> str
     """
     Replace accents by non accents characters when possible
 
@@ -37,27 +43,35 @@ def convert_accents(string: str) -> str:
     )
 
 
-def strip_characters(string: str, regex: str = r"[]") -> str:
+def strip_characters(string, regex=r"[]"):
+    # type: (str, str) -> str
     """
     Remove everything that is not in the list of allowed chars
+    Basically just a regex shortcut
     """
-    return re.sub(regex, "", string)
+    return re.sub(regex, "", string, re.UNICODE)
 
 
-def strip_special_characters(string: str, regex: str = r"[^a-zA-Z0-9 \n\._]") -> str:
+def strip_special_characters(string, regex=r"[^a-zA-Z0-9 \n\._]"):
+    # type: (str, str) -> str
     """
     Remove special characters except the ones allowed in the regex
+    Basically just a regex shortcut
     """
     return strip_characters(string, regex)
 
 
-def strip_non_alnum_characters(string: str, include_accents: bool = True) -> str:
+def strip_non_alnum_characters(string, keep_accents=True):
+    # type: (str, bool) -> str
     """
     Return only alphanumeric strings
+    Another regex shortcut
     """
-    if include_accents:
-        # Note: \W+ also includes '_' character
-        regex = r"\W+"
+    if keep_accents:
+        if sys.version_info[0] < 3:
+            regex = "[^a-zA-Z0-9À-ÖØ-öø-ÿ]"
+        else:
+            regex = "[^a-zA-Z0-9À-ÖØ-öø-ÿ]"
     else:
         regex = r"[^a-zA-Z0-9]"
 
